//**********************************************************************
// 	Project: 
//	File: exp_cfa.v
// 	Description: test CFA GW_2A_18
//	Author: wenxuan
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v2.0.0	| Wenxuan	| xx/xx/2021	| Initial version
//**********************************************************************


module exp_cfa (
// --- sym ---
	input    SYM_CLK,
// --- KEY ---
	input    KEY1,
// --- SW ----
	input    SW1,

	output			outvalid,
    output          line_sync_o,
    output          frame_sync_o,
	output [7:0]	R_dout,
	output [7:0]	G_dout,
	output [7:0]	B_dout
);

//**********************************************************************
// --- Parameter
//**********************************************************************
    localparam NUM_CYCLE = 1;
	localparam MEM_DEPTH = 64*20;
`define 	IMAGE_WIDTH 64	
`define 	IMAGE_HEIGHT 20	
//**********************************************************************
// --- Inter Signal Declaration
//**********************************************************************
    wire clk;
    // wire clk_100M;
	wire rstn;
	wire ce;

    assign clk = SYM_CLK;
    assign rstn = KEY1;
	assign ce=SW1;

    reg	inpvalid;
    reg line_sync;
    reg frame_sync;
//	wire signed [17:0]	din;
	
	reg pstart;
	reg [19:0] addr,addr_d,addr_2d;
    reg [9:0] cnt_cycle,cnt,cnt_line;

    reg [7:0] data_mem [0:MEM_DEPTH-1] /* synthesis syn_romstyle= "block_rom" */;
    reg [7:0] din;
//**********************************************************************
// --- Initial
//**********************************************************************

initial begin
           $readmemh("data.dat",data_mem);
end

//assign inpvalid  = (cnt_line<`IMAGE_HEIGHT)&&(cnt>4)&&(cnt<=`IMAGE_WIDTH+4) ? 1'b1 : 1'b0 ;
//assign line_sync = (cnt_line<`IMAGE_HEIGHT)&&(cnt== 5) ? 1'b1 : 1'b0 ;
//assign frame_sync= (cnt_line==0) &&( cnt==1 ) ? 1'b1: 1'b0;
    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin 
             inpvalid<= 0;
    end else if ((cnt_line<`IMAGE_HEIGHT)&&(cnt>4)&&(cnt<=`IMAGE_WIDTH+4))
             inpvalid<= 1;
        else 
             inpvalid<= 0;
    end
    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin 
             line_sync<= 0;
    end else if ((cnt_line<`IMAGE_HEIGHT)&&(cnt== 5))
             line_sync<= 1;
        else 
             line_sync<= 0;
    end
    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin 
             frame_sync<= 0;
    end else if ((cnt_line==0) &&( cnt==1 ) )
             frame_sync<= 1;
        else 
             frame_sync<= 0;
    end

    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin 
            cnt <= 10'h3fb;
    end else if (cnt==`IMAGE_WIDTH+19)
            cnt <= 0;
        else 
            cnt <= cnt+1;
    end
    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin 
            cnt_line <= 10'h3ff;
    end else if ((cnt_line==`IMAGE_HEIGHT+20)&&(cnt==0))
            cnt_line <= 0;
        else if(cnt==0)
            cnt_line <= cnt_line+1;
    end
    always @(posedge clk or negedge rstn) begin
        if(!rstn) addr <= 0;
        else if(ce) begin
			if(addr == MEM_DEPTH-1)
                addr <= 0;
            else if((cnt_line<`IMAGE_HEIGHT)&&(cnt>3)&&(cnt<=`IMAGE_WIDTH+3))
                addr <= addr + 1;
        end
    end
	
    always @(posedge clk or negedge rstn) begin
        if(!rstn) din <= 0;
        else if(ce ) begin
                din <= data_mem[addr];
        end
    end

//**********************************************************************
// --- Module: top 
// --- Description:	top module of project
//**********************************************************************
Color_Filter_Array_Interpolation_Top  Color_Filter_Array_Interpolation_Top(
       .clk(clk), 				// system clock
       .rstn(rstn),				// reset
    .frame_sync (frame_sync),
    .line_sync (line_sync),
    .inpvalid (inpvalid),
    .din (din),
    .frame_sync_o (frame_sync_o),
    .line_sync_o (line_sync_o),
    .outvalid(outvalid),
    .R_dout(R_dout),					// output data
    .G_dout(G_dout),					// output data
    .B_dout(B_dout)					// output data
);

	
 
endmodule
